/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file JSONBoxReader.cpp
 *  Implementation of parameter class JSONBoxReader.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 24.04.2014
 */

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/JSONBoxReader.h>
#include <fstream>

JSONBoxReader::JSONBoxReader(const std::string& name) :
		ParameteredObject(
			"JSONBoxReader", name,
			"<h2>Reads JSON box values</h2><br>Reads JSON box values and saves coordinates into std vectors."
		)
{

	ParameteredObject::_addOutputSlot(
		boxCoordinates, "boxCoordinates",
		"values from json file",
		"boxCoords");

	ParameteredObject::_addParameter(
		jsonFile, "jsonFile",
		"JSON file",
		"fileopen");

	ParameteredObject::_addParameter<bool>(
		verbose, "verbose",
		"Extended log",
		false,
		"bool");

}

void JSONBoxReader::execute() {
		
	// Read in file and convert it to json object
	std::ifstream file;
	file.open ( this->jsonFile() );
	std::string s((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
	file.close();

	json::Value json = json::Deserialize( s );

	// get root object
	json::Array arr = json.ToArray();

	int blat = 1;

	// iterate through first array
	for( size_t i = 0; i < arr.size(); i++ ){

		json::Object frameData = arr[i].ToObject();
		
		int frameId = frameData.find("frame")->second.ToInt();
		//std::string filename = entry.find("filename")->second.ToString();

		json::Array frameBoxes = frameData.find("results")->second.ToArray();
		
		//// save points of a box to vector
		for( size_t j = 0; j < frameBoxes.size(); j++ ){

			json::Object boxData = frameBoxes[j].ToObject();
			bool isDeleted = boxData.find("isDeleted")->second.ToBool();

			if(isDeleted)
				continue;

			json::Array boxCoords = boxData.find("data")->second.ToArray();
			
			float x1 = boxCoords[0].ToFloat() ;
			float y1 = boxCoords[1].ToFloat() ;
			float x2 = boxCoords[2].ToFloat() ;
			float y2 = boxCoords[3].ToFloat() ;

			std::pair< int, std::vector< float> > point;
			point.first = frameId;
			point.second.push_back(x1);
			point.second.push_back(y1);
			point.second.push_back(x2);
			point.second.push_back(y2);
			
			if( this->verbose() ) {
				sout << frameId << "\t" << x1 << "\t" << y1 << "\t" << x2 << "\t" << y2 << std::endl;
			}

			this->boxCoordinates().push_back( point );

		}

	}

}

// the following functions are needed
// for class JSONBoxReader to work as a charon plugin.
extern "C" jsonboxreader_DECLDIR ParameteredObject*
		create(const std::string& name, ParameteredObject::template_type) {
	return new JSONBoxReader(name);
}

extern "C" jsonboxreader_DECLDIR void destroy(ParameteredObject* b) {
	delete b;
}

/// Report build configuration to prevent linking of incompatibel runtime libs
extern "C" jsonboxreader_DECLDIR ParameteredObject::build_type getBuildType() {
#ifdef _DEBUG
	return ParameteredObject::DEBUG_BUILD;
#else
	return ParameteredObject::RELEASE_BUILD;
#endif
}
